from ipkiss3 import all as i3

TECH = i3.TECH

# We will define a RingResonator PCell which consists of
# a ring waveguide with evanescent coupling to a bus waveguide.


class RingResonator(i3.PCell):

    """A generic ring resonator class.

    It is defined by a circular waveguide evanescently coupled to a straight bus waveguide.
    """

    _name_prefix = "RINGRES"  # a prefix added to the unique identifier

    # 1. Defining the waveguide templates as WaveguideTemplateProperties
    wg_template = i3.WaveguideTemplateProperty(
        default=i3.TECH.PCELLS.WG.DEFAULT, doc="trace template used for the bus and the ring"
    )

    # 2. For the bus and the ring we use a ChildCellProperty.
    # The waveguide PCells themselves are created in the default method.
    bus = i3.ChildCellProperty(doc="bus waveguide")
    ring = i3.ChildCellProperty(doc="ring waveguide")

    # 3.Definition of the default values of the waveguide PCells
    def _default_ring(self):
        return i3.Waveguide(name=self.name + "_ring", trace_template=self.wg_template)

    def _default_bus(self):
        return i3.Waveguide(name=self.name + "_bus", trace_template=self.wg_template)

    class Layout(i3.LayoutView):

        ring_radius = i3.PositiveNumberProperty(default=i3.TECH.WG.BEND_RADIUS, doc="radius of the ring")
        coupler_spacing = i3.PositiveNumberProperty(
            default=i3.TECH.WG.DC_SPACING, doc="spacing between centerline of bus waveguide and ring waveguide"
        )

        def _default_ring(self):
            # 4.Definition of the default values of the layout view of the waveguide PCells
            ring_cell = self.cell.ring
            wg_template = self.wg_template

            ring_layout = ring_cell.get_default_view(i3.LayoutView)
            ring_layout.set(trace_template=wg_template, shape=i3.ShapeCircle(center=(0, 0), radius=self.ring_radius))

            return ring_layout

        def _default_bus(self):
            # 5. We do the same for the bus waveguide.
            bus_cell = self.cell.bus
            wg_template = self.wg_template
            r = self.ring_radius
            s = self.coupler_spacing

            bus_layout = bus_cell.get_default_view(i3.LayoutView)
            bus_layout.set(
                trace_template=wg_template, shape=[(-r, -r - s), (+r, -r - s)]
            )  # Using a shape made of list of coordinates

            return bus_layout

        def _generate_instances(self, insts):
            # 6. Placing a ring and bus instance in our layout.
            insts += i3.SRef(name="ring", reference=self.ring)
            insts += i3.SRef(name="bus", reference=self.bus)
            return insts

        def _generate_ports(self, ports):
            # 7. Add ports
            ports += self.instances["bus"].ports
            return ports


if __name__ == "__main__":
    print("This is not the main file. Run 'execute.py' in the same folder")
